@@ -41,10 +41,14 @@
  * modified 13-mar-2010 cjb	1. if yReadPause is set use it
  * modified 28-mar-2010 cjb	1. working on adding more DSP information
  * modified 27-jul-2010 cjb	1. added -P option to throttle commands sent to the inverter
+ * modified 21-sep-2010 cjb	1. fix using wrong param when displaying TransState message
+ *				2. pass TransState the command description so if it gets a non-zero status it can
+ *				   report for what command it got it
+ *				3. added reporting for "Last four alarms"
  *
  */
 
-char     VersionC[6] = "1.6.5";
+char     VersionC[6] = "1.6.7";
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -87,10 +91,10 @@ static char szSerBuffer[_szSerBufferLen];			/* serial read/write buffer */
 static WORD crcValue;
 
 /* local functions */
-static int TransState(int TransCode);
+static int TransState(int TransCode, char Desription[]);
 static int PrintModel(int ModelID);
-static long GetCEdata(int fdser, int yAddress, int opcode, int param);
-static float GetDSPdata(int fdser, int yAddress, int opcode, int param);
+static long GetCEdata(int fdser, int yAddress, int opcode, int param, char desription[]);
+static float GetDSPdata(int fdser, int yAddress, int opcode, int param, char desription[]);
 static int GetCountersData(int fdser, int yAddress, int param, char *uptime);
 static int Communicate(int fdser, int yAddress);
 static int ReadNextChar(int nfd, char *pChar, int timeout);
@@ -106,12 +110,12 @@ static unsigned long cvrtShort(char *Buffer);
 /*--------------------------------------------------------------------------
     TransState
 ----------------------------------------------------------------------------*/
-int TransState(int TransCode)
+int TransState(int TransCode, char Desription[])
 {
     if(bVerbose) fprintf(stderr, "Transmission State Check: %i\n",TransCode);
     if (TransCode == 0) return(1);
-    fprintf(outfp, "\nTransmission State: (%i)    %s\n",(int)szSerBuffer[aMState],FindString((int)szSerBuffer[aMState], szTransStates));
-    return(-1);
+    fprintf(outfp, "\nTransmission State: %2i Command: \"%s\" %s\n\n",TransCode,Desription,FindString(TransCode, szTransStates));
+    return(0);
 }
 
 
@@ -127,7 +131,7 @@ int CommCheck(int fdser, int yAddress)
     szSerBuffer[cCommand] = opGetVer;
     szSerBuffer[cCommandEnd] = '\0';
     nCnt = Communicate(fdser, yAddress);
-    if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetVer)) {
         ModelID = szSerBuffer[aPar1];
         if(bVerbose) {
             fprintf(stderr, "Model ID \"%c\" ",ModelID);
@@ -151,7 +155,7 @@ int GetState(int fdser, int yAddress)
     szSerBuffer[cCommand] = opGetState;
     szSerBuffer[cCommandEnd] = '\0';
     nCnt = Communicate(fdser, yAddress);
-    if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetState)) {
         fprintf(outfp, "\nGlobal State:          %s\n",FindString((int)szSerBuffer[aMState], szGlobalStates));
         fprintf(outfp, "Inverter State:        %s\n",FindString((int)szSerBuffer[aParam1], szInverterState));
         fprintf(outfp, "Channel 1 Dc/Dc State: %s\n",FindString((int)szSerBuffer[aParam2], szDcDcStatus));
@@ -163,6 +167,29 @@ int GetState(int fdser, int yAddress)
 }
 
 
+/*--------------------------------------------------------------------------
+    GetLastAlarms
+----------------------------------------------------------------------------*/
+int GetLastAlarms(int fdser, int yAddress)
+{
+    int nCnt;
+
+    strcpy(szSerBuffer, _clearCMD);             /* clear Aurora cmd string */
+    szSerBuffer[cAddress] = yAddress;           /* set inverter address */
+    szSerBuffer[cCommand] = opGetLastAlarms;
+    szSerBuffer[cCommandEnd] = '\0';
+    nCnt = Communicate(fdser, yAddress);
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetLastAlarms)) {
+        fprintf(outfp, "\nAlarm 1:               %s\n",FindString((int)szSerBuffer[aParam1], szAlarmState));
+        fprintf(outfp, "Alarm 2:               %s\n",FindString((int)szSerBuffer[aParam2], szAlarmState));
+        fprintf(outfp, "Alarm 3:               %s\n",FindString((int)szSerBuffer[aParam3], szAlarmState));
+        fprintf(outfp, "Alarm 4:               %s\n",FindString((int)szSerBuffer[aParam4], szAlarmState));
+        return(0);
+    }
+    return(-1);
+}
+
+
 /*--------------------------------------------------------------------------
     GetPN
 ----------------------------------------------------------------------------*/
@@ -228,7 +255,7 @@ int GetVer(int fdser, int yAddress)
     szSerBuffer[cParam1] = '.';
     nCnt = Communicate(fdser, yAddress);
     crcValue = crc16(szSerBuffer, 8);
-    if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetVer)) {
         PrintModel(szSerBuffer[aPar1]);
         switch (szSerBuffer[aPar2]) {
             case 'A':       fprintf(outfp, "%s -- ",aPar2A); break;
@@ -270,7 +297,7 @@ int GetConf(int fdser, int yAddress)
     szSerBuffer[cCommandEnd] = '\0';
     nCnt = Communicate(fdser, yAddress);
     crcValue = crc16(szSerBuffer, 8);
-    if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetConfig)) {
         switch (szSerBuffer[aConfCode]) {
             case ConfCode0:	fprintf(outfp, "\n%s\n",_ConfCode0); break;
             case ConfCode1:	fprintf(outfp, "\n%s\n",_ConfCode1); break;
@@ -318,7 +345,7 @@ int GetVerFW(int fdser, int yAddress)
     szSerBuffer[cCommand] = opGetVerFW;
     szSerBuffer[cCommandEnd] = '\0';
     nCnt = Communicate(fdser, yAddress);
-    if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetVerFW)) {
         fprintf(outfp, "\nFirmware: %c.%c.%c.%c\n",szSerBuffer[aRel3],szSerBuffer[aRel2],szSerBuffer[aRel1],szSerBuffer[aRel0]);
         return(0);
    }
@@ -340,7 +367,7 @@ int GetMfgDate(int fdser, int yAddress)
     szSerBuffer[cCommand] = opGetMfg;
     szSerBuffer[cCommandEnd] = '\0';
     nCnt = Communicate(fdser, yAddress);
-    if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetMfg)) {
         MfgWeek[0] = szSerBuffer[aWeekH];
         MfgWeek[1] = szSerBuffer[aWeekL];
         MfgYear[0] = szSerBuffer[aYearH];
@@ -361,15 +388,15 @@ int GetCE(int fdser, int yAddress)
     long DAILY, WEEKLY, LAST7DAYS, MONTHLY, YEARLY, TOTAL, PARTIAL;
 
     if(bVerbose) fprintf(stderr, "\nAttempting to get Partial Energy value ");
-    if((PARTIAL = GetCEdata(fdser,yAddress,opGetCE,CEpar6)) < 0) return(-1);
-    if((DAILY = GetCEdata(fdser,yAddress,opGetCE,CEpar0)) < 0) return(-1);
-    if((WEEKLY = GetCEdata(fdser,yAddress,opGetCE,CEpar1)) < 0) return(-1);
-//    if((LAST7DAYS = GetCEdata(fdser,yAddress,opGetCE,CEpar2)) < 0) return(-1);
+    if((PARTIAL = GetCEdata(fdser,yAddress,opGetCE,CEpar6,_opGetCE)) < 0) return(-1);
+    if((DAILY = GetCEdata(fdser,yAddress,opGetCE,CEpar0,_opGetCE)) < 0) return(-1);
+    if((WEEKLY = GetCEdata(fdser,yAddress,opGetCE,CEpar1,_opGetCE)) < 0) return(-1);
+//    if((LAST7DAYS = GetCEdata(fdser,yAddress,opGetCE,CEpar2,_opGetCE)) < 0) return(-1);
     LAST7DAYS = 0.0;	/* do this for now since this has been dropped in the v2.3 Communications Protocol doc	*/
 			/* placeholder for the -c option for now						*/
-    if((MONTHLY = GetCEdata(fdser,yAddress,opGetCE,CEpar3)) < 0) return(-1);
-    if((YEARLY = GetCEdata(fdser,yAddress,opGetCE,CEpar4)) < 0) return(-1);
-    if((TOTAL = GetCEdata(fdser,yAddress,opGetCE,CEpar5)) < 0) return(-1);
+    if((MONTHLY = GetCEdata(fdser,yAddress,opGetCE,CEpar3,_opGetCE)) < 0) return(-1);
+    if((YEARLY = GetCEdata(fdser,yAddress,opGetCE,CEpar4,_opGetCE)) < 0) return(-1);
+    if((TOTAL = GetCEdata(fdser,yAddress,opGetCE,CEpar5,_opGetCE)) < 0) return(-1);
 
     if(bColumns) {
         fprintf(outfp, "%11.3f  %11.3f  %11.3f  %11.3f  %11.3f  %11.3f  %11.3f  ",DAILY/1000.0,WEEKLY/1000.0,LAST7DAYS/1000.0,MONTHLY/1000.0,YEARLY/1000.0,TOTAL/1000.0,PARTIAL/1000.0);
@@ -408,7 +435,7 @@ int GetCE(int fdser, int yAddress)
 /*--------------------------------------------------------------------------
     GetCEdata
 ----------------------------------------------------------------------------*/
-long GetCEdata(int fdser, int yAddress, int opcode, int param)
+long GetCEdata(int fdser, int yAddress, int opcode, int param, char desription[])
 {
     int nCnt;
     unsigned long paramValLong;
@@ -419,7 +446,7 @@ long GetCEdata(int fdser, int yAddress, int opcode, int param)
     szSerBuffer[cParam1] = param;
     szSerBuffer[cCommandEnd+1] = '\0';
     nCnt = Communicate(fdser, yAddress);
-    if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],desription)) {
         paramValLong = cvrtLong(szSerBuffer);
         if(bVerbose) fprintf(stderr, "value        %12lu\n",(unsigned long)paramValLong);
     }
@@ -437,18 +464,18 @@ int GetDSP(int fdser, int yAddress)
 {
     float GVR, GCR, GPR, GPRC=0.0, FRQ, INVeff=0.0, INVeffC=0.0, INVtemp, ENVtemp, STR1V, STR1C, STR2V, STR2C, PVpwr;
 
-    if((FRQ = GetDSPdata(fdser,yAddress,opGetDSP,ToM4)) < 0) return(-1);
-    if((GVR = GetDSPdata(fdser,yAddress,opGetDSP,ToM1)) < 0) return(-1);
-    if((GCR = GetDSPdata(fdser,yAddress,opGetDSP,ToM2)) < 0) return(-1);
-    if((GPR = GetDSPdata(fdser,yAddress,opGetDSP,ToM3)) < 0) return(-1);
+    if((FRQ = GetDSPdata(fdser,yAddress,opGetDSP,ToM4,_opGetDSP)) < 0) return(-1);
+    if((GVR = GetDSPdata(fdser,yAddress,opGetDSP,ToM1,_opGetDSP)) < 0) return(-1);
+    if((GCR = GetDSPdata(fdser,yAddress,opGetDSP,ToM2,_opGetDSP)) < 0) return(-1);
+    if((GPR = GetDSPdata(fdser,yAddress,opGetDSP,ToM3,_opGetDSP)) < 0) return(-1);
 
-    if((STR1V = GetDSPdata(fdser,yAddress,opGetDSP,ToM23)) < 0) return(-1);
-    if((STR1C = GetDSPdata(fdser,yAddress,opGetDSP,ToM25)) < 0) return(-1);
-    if((STR2V = GetDSPdata(fdser,yAddress,opGetDSP,ToM26)) < 0) return(-1);
-    if((STR2C = GetDSPdata(fdser,yAddress,opGetDSP,ToM27)) < 0) return(-1);
+    if((STR1V = GetDSPdata(fdser,yAddress,opGetDSP,ToM23,_opGetDSP)) < 0) return(-1);
+    if((STR1C = GetDSPdata(fdser,yAddress,opGetDSP,ToM25,_opGetDSP)) < 0) return(-1);
+    if((STR2V = GetDSPdata(fdser,yAddress,opGetDSP,ToM26,_opGetDSP)) < 0) return(-1);
+    if((STR2C = GetDSPdata(fdser,yAddress,opGetDSP,ToM27,_opGetDSP)) < 0) return(-1);
 
-    if((INVtemp = GetDSPdata(fdser,yAddress,opGetDSP,ToM21)) < 0) return(-1);
-    if((ENVtemp = GetDSPdata(fdser,yAddress,opGetDSP,ToM22)) < 0) return(-1);
+    if((INVtemp = GetDSPdata(fdser,yAddress,opGetDSP,ToM21,_opGetDSP)) < 0) return(-1);
+    if((ENVtemp = GetDSPdata(fdser,yAddress,opGetDSP,ToM22,_opGetDSP)) < 0) return(-1);
 
     if(bCalcGridPwr) GPRC = GVR * GCR;
 
@@ -504,37 +531,37 @@ int GetDSPExtended(int fdser, int yAddress)
     float VB,ILD,ILI,P1,P2,GV,GF,IR,VBD,AGV,VBM,PP,PPT,GVn,WGF,VBp,VBm,ST,AT,HT,T1,T2,T3,F1,F2,F3,F4,F5,PSL,RRB,VPm;
     char note[3];
 
-    if((VB = GetDSPdata(fdser,yAddress,opGetDSP,ToM5)) < 0) return(-1);
-    if((ILD = GetDSPdata(fdser,yAddress,opGetDSP,ToM6)) < 0) return(-1);
-    if((ILI = GetDSPdata(fdser,yAddress,opGetDSP,ToM7)) < 0) return(-1);
-    if((P1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM8)) < 0) return(-1);
-    if((P2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM9)) < 0) return(-1);
-    if((GV = GetDSPdata(fdser,yAddress,opGetDSP,ToM28)) < 0) return(-1);
-    if((GF = GetDSPdata(fdser,yAddress,opGetDSP,ToM29)) < 0) return(-1);
-    if((IR = GetDSPdata(fdser,yAddress,opGetDSP,ToM30)) < 0) return(-1);
-    if((VBD = GetDSPdata(fdser,yAddress,opGetDSP,ToM31)) < 0) return(-1);
-    if((AGV = GetDSPdata(fdser,yAddress,opGetDSP,ToM32)) < 0) return(-1);
-    if((VBM = GetDSPdata(fdser,yAddress,opGetDSP,ToM33)) < 0) return(-1);
-    if((PP = GetDSPdata(fdser,yAddress,opGetDSP,ToM34)) < 0) return(-1);
-    if((PPT = GetDSPdata(fdser,yAddress,opGetDSP,ToM35)) < 0) return(-1);
-    if((GVn = GetDSPdata(fdser,yAddress,opGetDSP,ToM36)) < 0) return(-1);
-    if((WGF = GetDSPdata(fdser,yAddress,opGetDSP,ToM37)) < 0) return(-1);
-    if((VBp = GetDSPdata(fdser,yAddress,opGetDSP,ToM45)) < 0) return(-1);
-    if((VBm = GetDSPdata(fdser,yAddress,opGetDSP,ToM46)) < 0) return(-1);
-    if((ST = GetDSPdata(fdser,yAddress,opGetDSP,ToM47)) < 0) return(-1);
-    if((AT = GetDSPdata(fdser,yAddress,opGetDSP,ToM48)) < 0) return(-1);
-    if((HT = GetDSPdata(fdser,yAddress,opGetDSP,ToM49)) < 0) return(-1);
-    if((T1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM50)) < 0) return(-1);
-    if((T2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM51)) < 0) return(-1);
-    if((T3 = GetDSPdata(fdser,yAddress,opGetDSP,ToM52)) < 0) return(-1);
-    if((F1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM52)) < 0) return(-1);
-    if((F2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM54)) < 0) return(-1);
-    if((F3 = GetDSPdata(fdser,yAddress,opGetDSP,ToM55)) < 0) return(-1);
-    if((F4 = GetDSPdata(fdser,yAddress,opGetDSP,ToM56)) < 0) return(-1);
-    if((F5 = GetDSPdata(fdser,yAddress,opGetDSP,ToM57)) < 0) return(-1);
-    if((PSL = GetDSPdata(fdser,yAddress,opGetDSP,ToM58)) < 0) return(-1);
-    if((RRB = GetDSPdata(fdser,yAddress,opGetDSP,ToM59)) < 0) return(-1);
-    if((VPm = GetDSPdata(fdser,yAddress,opGetDSP,ToM60)) < 0) return(-1);
+    if((VB = GetDSPdata(fdser,yAddress,opGetDSP,ToM5,_opGetDSP)) < 0) return(-1);
+    if((ILD = GetDSPdata(fdser,yAddress,opGetDSP,ToM6,_opGetDSP)) < 0) return(-1);
+    if((ILI = GetDSPdata(fdser,yAddress,opGetDSP,ToM7,_opGetDSP)) < 0) return(-1);
+    if((P1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM8,_opGetDSP)) < 0) return(-1);
+    if((P2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM9,_opGetDSP)) < 0) return(-1);
+    if((GV = GetDSPdata(fdser,yAddress,opGetDSP,ToM28,_opGetDSP)) < 0) return(-1);
+    if((GF = GetDSPdata(fdser,yAddress,opGetDSP,ToM29,_opGetDSP)) < 0) return(-1);
+    if((IR = GetDSPdata(fdser,yAddress,opGetDSP,ToM30,_opGetDSP)) < 0) return(-1);
+    if((VBD = GetDSPdata(fdser,yAddress,opGetDSP,ToM31,_opGetDSP)) < 0) return(-1);
+    if((AGV = GetDSPdata(fdser,yAddress,opGetDSP,ToM32,_opGetDSP)) < 0) return(-1);
+    if((VBM = GetDSPdata(fdser,yAddress,opGetDSP,ToM33,_opGetDSP)) < 0) return(-1);
+    if((PP = GetDSPdata(fdser,yAddress,opGetDSP,ToM34,_opGetDSP)) < 0) return(-1);
+    if((PPT = GetDSPdata(fdser,yAddress,opGetDSP,ToM35,_opGetDSP)) < 0) return(-1);
+    if((GVn = GetDSPdata(fdser,yAddress,opGetDSP,ToM36,_opGetDSP)) < 0) return(-1);
+    if((WGF = GetDSPdata(fdser,yAddress,opGetDSP,ToM37,_opGetDSP)) < 0) return(-1);
+    if((VBp = GetDSPdata(fdser,yAddress,opGetDSP,ToM45,_opGetDSP)) < 0) return(-1);
+    if((VBm = GetDSPdata(fdser,yAddress,opGetDSP,ToM46,_opGetDSP)) < 0) return(-1);
+    if((ST = GetDSPdata(fdser,yAddress,opGetDSP,ToM47,_opGetDSP)) < 0) return(-1);
+    if((AT = GetDSPdata(fdser,yAddress,opGetDSP,ToM48,_opGetDSP)) < 0) return(-1);
+    if((HT = GetDSPdata(fdser,yAddress,opGetDSP,ToM49,_opGetDSP)) < 0) return(-1);
+    if((T1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM50,_opGetDSP)) < 0) return(-1);
+    if((T2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM51,_opGetDSP)) < 0) return(-1);
+    if((T3 = GetDSPdata(fdser,yAddress,opGetDSP,ToM52,_opGetDSP)) < 0) return(-1);
+    if((F1 = GetDSPdata(fdser,yAddress,opGetDSP,ToM52,_opGetDSP)) < 0) return(-1);
+    if((F2 = GetDSPdata(fdser,yAddress,opGetDSP,ToM54,_opGetDSP)) < 0) return(-1);
+    if((F3 = GetDSPdata(fdser,yAddress,opGetDSP,ToM55,_opGetDSP)) < 0) return(-1);
+    if((F4 = GetDSPdata(fdser,yAddress,opGetDSP,ToM56,_opGetDSP)) < 0) return(-1);
+    if((F5 = GetDSPdata(fdser,yAddress,opGetDSP,ToM57,_opGetDSP)) < 0) return(-1);
+    if((PSL = GetDSPdata(fdser,yAddress,opGetDSP,ToM58,_opGetDSP)) < 0) return(-1);
+    if((RRB = GetDSPdata(fdser,yAddress,opGetDSP,ToM59,_opGetDSP)) < 0) return(-1);
+    if((VPm = GetDSPdata(fdser,yAddress,opGetDSP,ToM60,_opGetDSP)) < 0) return(-1);
 
     note[0] = '\0';
     if(bColumns) {
@@ -591,17 +618,17 @@ int GetDSP3Phase(int fdser, int yAddress)
     float VBD,GVPn,GVPr,GVPs,GVPt,GCPr,GCPs,GCPt,FRQPr,FRQPs,FRQPt;
     char note[3];
 
-    if((VBD = GetDSPdata(fdser,yAddress,opGetDSP,ToM31)) < 0) return(-1);
-    if((GVPn = GetDSPdata(fdser,yAddress,opGetDSP,ToM38)) < 0) return(-1);
-    if((GVPr = GetDSPdata(fdser,yAddress,opGetDSP,ToM61)) < 0) return(-1);
-    if((GVPs = GetDSPdata(fdser,yAddress,opGetDSP,ToM62)) < 0) return(-1);
-    if((GVPt = GetDSPdata(fdser,yAddress,opGetDSP,ToM63)) < 0) return(-1);
-    if((GCPr = GetDSPdata(fdser,yAddress,opGetDSP,ToM39)) < 0) return(-1);
-    if((GCPs = GetDSPdata(fdser,yAddress,opGetDSP,ToM40)) < 0) return(-1);
-    if((GCPt = GetDSPdata(fdser,yAddress,opGetDSP,ToM41)) < 0) return(-1);
-    if((FRQPr = GetDSPdata(fdser,yAddress,opGetDSP,ToM42)) < 0) return(-1);
-    if((FRQPs = GetDSPdata(fdser,yAddress,opGetDSP,ToM43)) < 0) return(-1);
-    if((FRQPt = GetDSPdata(fdser,yAddress,opGetDSP,ToM44)) < 0) return(-1);
+    if((VBD = GetDSPdata(fdser,yAddress,opGetDSP,ToM31,_opGetDSP)) < 0) return(-1);
+    if((GVPn = GetDSPdata(fdser,yAddress,opGetDSP,ToM38,_opGetDSP)) < 0) return(-1);
+    if((GVPr = GetDSPdata(fdser,yAddress,opGetDSP,ToM61,_opGetDSP)) < 0) return(-1);
+    if((GVPs = GetDSPdata(fdser,yAddress,opGetDSP,ToM62,_opGetDSP)) < 0) return(-1);
+    if((GVPt = GetDSPdata(fdser,yAddress,opGetDSP,ToM63,_opGetDSP)) < 0) return(-1);
+    if((GCPr = GetDSPdata(fdser,yAddress,opGetDSP,ToM39,_opGetDSP)) < 0) return(-1);
+    if((GCPs = GetDSPdata(fdser,yAddress,opGetDSP,ToM40,_opGetDSP)) < 0) return(-1);
+    if((GCPt = GetDSPdata(fdser,yAddress,opGetDSP,ToM41,_opGetDSP)) < 0) return(-1);
+    if((FRQPr = GetDSPdata(fdser,yAddress,opGetDSP,ToM42,_opGetDSP)) < 0) return(-1);
+    if((FRQPs = GetDSPdata(fdser,yAddress,opGetDSP,ToM43,_opGetDSP)) < 0) return(-1);
+    if((FRQPt = GetDSPdata(fdser,yAddress,opGetDSP,ToM44,_opGetDSP)) < 0) return(-1);
 
     note[0] = '\0';
     if(bColumns) {
@@ -632,7 +659,7 @@ int GetDSP3Phase(int fdser, int yAddress)
 /*--------------------------------------------------------------------------
     GetDSPdata
 ----------------------------------------------------------------------------*/
-float GetDSPdata(int fdser, int yAddress, int opcode, int param)
+float GetDSPdata(int fdser, int yAddress, int opcode, int param, char desription[])
 {
     int nCnt;
     float *paramValFloat;
@@ -643,7 +670,7 @@ float GetDSPdata(int fdser, int yAddress, int opcode, int param)
     szSerBuffer[cParam1] = param;
     szSerBuffer[cCommandEnd+1] = '\0';
     nCnt = Communicate(fdser, yAddress);
-    if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],desription)) {
 
         paramValFloat = cvrtFloat(szSerBuffer);
         if(bVerbose) fprintf(stderr, "value     %12.6f\n",*paramValFloat);
@@ -668,7 +695,7 @@ int GetJoules(int fdser, int yAddress)
     szSerBuffer[cCommand] = opGetEnergy10Sec;
     szSerBuffer[cCommandEnd] = '\0';
     nCnt = Communicate(fdser, yAddress);
-    if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetEnergy10Sec)) {
         Joules = cvrtShort(szSerBuffer);
         if(bVerbose) fprintf(stderr, "Joules       %12lu\n",(unsigned long)Joules);
         fprintf(outfp, "\nEnergy in the last 10 seconds (Joules) : %11.6f\n",(unsigned long)Joules*0.319509);
@@ -698,7 +725,7 @@ int GetTime(int fdser, int yAddress)
         nCnt = Communicate(fdser, yAddress);
     }
     if (nCnt > 0) {
-        if (! TransState((int)szSerBuffer[aState])) return(-1);
+        if (! TransState((int)szSerBuffer[aState],_opGetTime)) return(-1);
         if(bVerbose) {
             timeValLong = (time_t)TimeBase;
             fprintf(stderr, "\nTimeBase     %12lu\n",(time_t)TimeBase);
@@ -778,7 +805,7 @@ int SetTime(int fdser, int yAddress)
     szSerBuffer[cCommandEnd+4] = '\0';
     nCnt = Communicate(fdser, yAddress);
     if (nCnt > 0) {
-        if (! TransState((int)szSerBuffer[aState])) return(-1);
+        if (! TransState((int)szSerBuffer[aState],_opSetTime)) return(-1);
         printf("\nInverter date/time set.");
         return(0);
    }
@@ -823,7 +850,7 @@ int GetCountersData(int fdser, int yAddress, int param, char *uptime)
     szSerBuffer[cParam1] = param;
     szSerBuffer[cCommandEnd+1] = '\0';
     nCnt = Communicate(fdser, yAddress);
-    if (nCnt > 0 && TransState((int)szSerBuffer[aState])) {
+    if (nCnt > 0 && TransState((int)szSerBuffer[aState],_opGetCounters)) {
         paramValPtr = cvrtLong(szSerBuffer);
         if(bVerbose) fprintf(stderr, "value        %12lu\n",(unsigned long)paramValPtr);
     }
