@@ -27,16 +27,18 @@
  *
  */
 
-#define	VersionNH	"1.6.1"
+#define	VersionNH	"1.6.7"
 
 /* # 50 State Request */
 
 #define	opGetState	50
+#define _opGetState "State Request"
 
 
 /* # 52 P/N Reading */
 
 #define opGetPN		52
+#define _opGetPN "P/N Reading"
 
 #define	aChar6	0
 #define aChar5  1
@@ -49,6 +51,7 @@
 /* # 58 Version Reading */
 
 #define opGetVer	58
+#define _opGetVer "Version Reading"
 
 #define aPar1	2
 #define aPar1i	"Aurora 2 kW indoor"
@@ -87,6 +90,7 @@
 /* # 59 Measure request to the DSP - Type of Measure */
 
 #define	opGetDSP	59
+#define _opGetDSP "Measure request to the DSP"
 
 #define	ToM1	1		/* Grid Voltage (Global) */
 #define ToM2	2		/* Grid Currrent (Global) */
@@ -205,11 +209,13 @@
 /* # 63 Serial Number */
 
 #define	opGetSN		63
+#define _opGetSN "Serial Number"
 
 
 /* # 65 Manufacturing Week and Year */
 
 #define opGetMfg	65
+#define _opGetMfg "Manufacturing Week and Year"
 
 #define	aWeekH	2
 #define aWeekL	3
@@ -220,16 +226,19 @@
 /* # 70 Get Time/Date */
 
 #define opGetTime	70
+#define _opGetTime "Get Time/Date"
 
 
 /* # 71 Set Time/Date */
 
 #define opSetTime	71
+#define _opSetTime "Set Time/Date"
 
 
 /* # 72 Firmware Release */
 
 #define opGetVerFW	72
+#define _opGetVerFW "Firmware Release"
 
 #define aRel3	2
 #define aRel2   3
@@ -240,6 +249,7 @@
 /* # 76 Energy cumulated in the last 10 seconds */
 
 #define	opGetEnergy10Sec	76
+#define _opGetEnergy10Sec "Energy cumulated in the last 10 seconds"
 
 #define	aEnergyH	2
 #define	aEnergyL	3
@@ -248,6 +258,7 @@
 /* # 77 System Configuration */
 
 #define opGetConfig	77
+#define _opGetConfig "System Configuration"
 
 #define	aConfCode	2
 
@@ -263,6 +274,7 @@
 /* # 78 Cumulated Energy */
 
 #define	opGetCE		78
+#define _opGetCE "Cumulated Energy"
 
 #define CEpar0	0		/* Daily Energy */
 #define CEpar1  1		/* Weekly Energy */
@@ -283,6 +295,7 @@
 /* # 80 Time Counter */
 
 #define	opGetCounters	80
+#define _opGetCounters "Time Counter"
 
 #define	cTotalRun	0
 #define cPartialRun	1
@@ -294,5 +307,11 @@
 #define _cTotalGrid	"Total Time With Grid Connection"
 #define _cResetPartial	"Reset of Partial (Time & Energy)"
 
+/* # 86 Last four alarms */
+
+#define opGetLastAlarms	86
+#define _opGetLastAlarms "Last four alarms"
+
+/* return codes same as alarms from # 50 State request */
 
 
