@@ -44,10 +44,11 @@
  *				3. changed -U seconds to wait to milli-seconds to wait
  * modified 28-mar-2010 cjb	1. working on adding more DSP information
  * modified 27-jul-2010 cjb	1. added -P option to throttle commands sent to the inverter
+ * modified 21-sep-2010 cjb	1. added -A option to read "Last four alarms"
  *
  */
 
-static char	VersionM[] = "1.6.6";
+static char	VersionM[] = "1.6.7";
 char     VersionC[6];
 
 #include <syscall.h>
@@ -121,6 +122,7 @@ static BOOL bGetCount = FALSE;			/* Get Time Counters */
 static BOOL bSetTime = FALSE;			/* Set time flag */
 static BOOL bVersion = FALSE;			/* Display program version */
 static BOOL bGetTime;
+static BOOL bGetLastAlarms = FALSE;		/* Get last four alarms */
 static BOOL bFileError = FALSE;
 static unsigned char yDelay;			/* Read wait time */
 static int yLockWait = 0;			/* Seconds to wait to lock serial port */
@@ -162,6 +164,7 @@ int main(int argc, char *argv[])
         printf("\nSee http://www.curtronics.com/Solar/ for Solar Site example\n\n");
         printf("Usage: %s [Options] Device\t\t\tv%s\n\n",ProgramName,VersionM);
         printf("Options:\n");
+        printf(" -A, --last-alarms              Get last four alarms (once displayed FIFO queue is cleared)\n");
         printf(" -b, --verbose                  Verbose mode\n");
         printf(" -C, --calc-value=num           Calculate $ value using num * KWh\n");
         printf(" -c, --columnize                Output data in columns --> for -d, -e, -D, & -E options only\n");
@@ -426,6 +429,9 @@ int main(int argc, char *argv[])
 
             if (rc == 0 && bGetCount)
                 rc |= GetCounters(fdser,yAddress);
+
+            if (rc == 0 && bGetLastAlarms)
+                rc |= GetLastAlarms(fdser,yAddress);
         }
 
         if (rc == 0 && bGetDSP)
@@ -575,6 +581,7 @@ int GetParms(int argc, char *argv[])
      /* options descriptor */
     static struct option longopts[] = {
         { "address",		required_argument,	0,	'a' },
+	{ "last-alarms",	no_argument,		0,	'A' },
         { "verbose",		no_argument,		0,	'b' },
 	{ "calc-value",		required_argument,	0,	'C' },
         { "columnize",		no_argument,		0,	'c' },
@@ -619,12 +626,13 @@ int GetParms(int argc, char *argv[])
     if(argc == 1)
         return 0;				/* no parms at all */
 
-    while ((c = getopt_long(argc, argv, "a:bC:cDdEefgHhijl:mno:P:pR:rSsTtVvU:uw:xY:y", longopts, NULL )) != EOF) {
+    while ((c = getopt_long(argc, argv, "a:AbC:cDdEefgHhijl:mno:P:pR:rSsTtVvU:uw:xY:y", longopts, NULL )) != EOF) {
         switch (c) {
             case 'a':
                 /* Inverter address */
                 yAddress = atoi(optarg);
                 break;
+            case 'A': bGetLastAlarms     = TRUE; break;
             case 'b': bVerbose     = TRUE; break;
             case 'C':
                 yCost = atof(optarg);
